#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <mqueue.h>
#include <string.h>

void errExit(const char *message)
{
	perror(message);
	exit(EXIT_FAILURE);
}

int main()
{
	ssize_t taille_lue = 0;
	const size_t buffer_size = 2048;
	char buffer[buffer_size];
	mqd_t file_message;
	struct mq_attr attr;
	int ret = 0;

	// On s'assure que tout est bien initialisé à 0 :
	memset(&attr, 0, sizeof(struct mq_attr));
	// Définition des attributs pour notre file de message :
	attr.mq_maxmsg = 10;
	attr.mq_msgsize = 2048;

	printf("Création de la file de message...\n");
	file_message = mq_open("/filemessage", O_CREAT | O_WRONLY, S_IRUSR | S_IWUSR, &attr);
	if (file_message == (mqd_t) -1) {
		errExit("Erreur lors de la création ou de l'ouverture de la file de message");
	}
	printf("File de message ouverte avec succès.\n");

	printf("Lecture de l'entrée standard en boucle. ^D pour terminer :\n");
	while (1) {
		taille_lue = read(STDIN_FILENO, buffer, buffer_size);
		if (taille_lue == -1) {
			errExit("Erreur lors de la lecture de l'entrée standard");
		} else if (taille_lue == 0) {
			ret = mq_send(file_message, buffer, 0, 0);
			if (ret == -1) {
				errExit("Erreur lors de l'envoie dans la file de message");
			}
			break;
		} else {
			ret = mq_send(file_message, buffer, (unsigned int)taille_lue, rand() % 10);
			if (ret == -1) {
				errExit("Erreur lors de l'envoie dans la file de message");
			}
		}
	}
	printf("Fin de la lecture.\n");

	printf("Suppression de la file de message...\n");
	ret = mq_unlink("/filemessage");
	if (ret != 0) {
		errExit("mq_unlink");
	}
	printf("File de message supprimée.\n");

	return EXIT_SUCCESS;
}
