#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

void errExit(const char *message)
{
	perror(message);
	exit(EXIT_FAILURE);
}

int main()
{
	int ret = 0, tube = 0;
	ssize_t taille_lue = 0, taille_ecrite = 0, retour_write = 0;
	const size_t buffer_size = 256;
	char buffer[buffer_size];
	
	printf("Création du tube...\n");
	ret = mkfifo("montube", S_IRUSR | S_IWUSR);
	if (ret != 0) {
		switch (errno) {
			case EEXIST:
				printf("Le tube existe déjà.\n");
				break;
			default:
				errExit("Erreur lors de la création du tube");
		}
	}
	printf("Tube créé avec succès.\n");

	printf("Ouverture du tube en écriture...\n");
	tube = open("montube", O_WRONLY);
	if (tube == -1) {
		errExit("Erreur lors de l'ouverture du tube");
	}
	printf("Ouverture du tube avec succès.\n");

	printf("Lecture de l'entrée standard en boucle. ^D pour terminer :\n");
	while (1) {
		taille_lue = read(STDIN_FILENO, buffer, buffer_size);
		if (taille_lue == -1) {
			errExit("Erreur lors de la lecture de l'entrée standard");
		} else if (taille_lue == 0) {
			break;
		} else {
			taille_ecrite = 0;
			while (taille_ecrite < taille_lue) {
				retour_write = write(tube, buffer + taille_ecrite, taille_lue - taille_ecrite);
				if (retour_write == -1) {
					errExit("Erreur lors de l'écriture dans le tube");
				}
				taille_ecrite += retour_write;
			}
		}
	}
	printf("Fin de la lecture.\n");

	ret = unlink("montube");
	if (ret == -1) {
		errExit("unlink");
	}

	return EXIT_SUCCESS;
}
